/*
Dieses Programm ermglicht es ber SPI mit dem iLCD zu komunizieren.
Ein Teil der seriellen iLCD Befehle ist als Funktion vorhanden aber lange nicht alle.
Verwendete Hardware:
 - iLCD DPP-CTS2432 mit DPC6050 Controller von www.demmel.com
 - C Board "Savvy128 V1.3 ATmega128 USB RS232 RS485 Echtzeituhr SD-Karte" von www.chip45.com
 - Inclusive dem "SavvyDISP V1.5 LCD Tasten Modul" 
*/


#include <avr/io.h>
#include <util/delay.h>
#include <iLCD_SPI.h>





//**************************************
//SPI Funktionen
//**************************************


//****************************************************************************
void SPI_init(void)
{
  // Set MOSI, SCK and SSEL as output, all others input 
  DDRB = (1<<MOSI)|(1<<SCK)|(1<<SiLCD)|(1<<SS);
  PORTB |= (1<<SiLCD) | (1<<SS);		// iLCD nicht enabled (high) und SD Karte ausschalten (besonders MISO von SD Karte in Tristate schalten)
  // Enable SPI, Master, set clock rate fck/16 = 1MHz
  SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);  // Clock idel = 0, Datenbernahme mit steigender Flanke
}




//****************************************************************************
void SPI_putc(char cData)
{
  PORTB |= (1<<SS);		// SD Karte ausschalten falls nicht schon aus.
  PORTB &= ~(1<<SiLCD);		// iLCD enablen (low)

  // Start transmission
  SPDR = cData;
  // Wait for transmission complete 
  while(!(SPSR & (1<<SPIF)));

  PORTB |= (1<<SiLCD);		// iLCD nicht enabled (high)
  _delay_us(4);				// iLCD Zeit fr die Datenverarbeitung lassen
}




//****************************************************************************
char SPI_getc(void)
{
  char cData;
 
  PORTB |= (1<<SS);		// SD Karte ausschalten falls nicht schon aus.
  PORTB &= ~(1<<SiLCD);		// iLCD enablen (low)

  SPI_putc(0xAA);		// Start Byte
  SPI_putc(0x04);		// Command "Block Read"
  SPI_putc(0x1);		// Anzahl der zu lesenden Bytes
  SPI_putc(0x00);		// iLCD Schaltet auf schreiben um
  SPI_putc(0x00);		// Byte senden zur Takterzeugung

  cData = SPDR;			// SPI Buffer auslesen

  PORTB |= (1<<SiLCD);		// iLCD nicht enabled (high)

  return cData;
}




//******************************************************************************
char iLCD_CMD(const char *command)
{
  char receive; 
  unsigned char count;	

  SPI_putc(0xAA);		// Start Byte
  SPI_putc(0x05);		// Block Write, speziell fr ISP bertragung notwendig
  SPI_putc(*command);		// Datenlnge ankndigen


  for ( count=*command++; count>0; count--)
    {
	  SPI_putc(*command++); 
    }

  SPI_putc(0x00);


  // checken ob alles gut angekommen ist durch warten auf <ACK> (0x06)
  for ( unsigned int i=0; i<5000; i++)				// ca. 0,5s Zeit lassen um den Befehl aus zu fhren
  	{
	  receive = SPI_getc();
	  if(receive == 0x06) return 0 ;		// es hat alles geklappt
	  if(receive == 0x15) return 1;			// hier ist was schief gelaufen <NACK>

	 _delay_us(100);
	}

  return 2;  // bertragung fehlerhaft weil kein <ACK> innerhalb angemessener Zeit bekommen

}










//*************************************
// iLCD Befehle
//*************************************


//****************************************************************************
void ResetAll(void)
{
  char data[] = {
  				 1		,
				 '!'	,
				};

  iLCD_CMD(data);
}//end



//*****************************************************************************
void SetCursorPosition(int x, int y)
{
  char cmd[] = {
  				6,			// anzahl der zu sendenen Bytes
  				'C',		// kommando
				'K',		// kommando
				x>>8,		// high Byte von x
				x,			// low byte von x
				y>>8,		// high Byte von x
				y,			// low byte von x
				};
  
  iLCD_CMD(cmd);
} // end



//*****************************************************************************
void WriteText(char *string)
{
  unsigned char count=0, cmd[128];
  
  cmd[1] = 'D';
  cmd[2] = 'T';
 
  count = 3;

  // Stringlnge auszhlen
  while(*string)	// abarbeiten bis null Zeichen kommt
  {
	cmd[count++] = *string++;	// String gleich einarbeiten
  }
  
  cmd[count] = 0;		// String Abschlusszeichen
  cmd[0] = count;		// Anzahl der zu bermittelden Zeichen eintragen


  iLCD_CMD(cmd);
}




//*****************************************************************************
void SetLineStyle(char style)
{
  char data[] = { 
				3,
				'C',
				'L',
				style
				};

  iLCD_CMD(data);
}//end




//****************************************************************************
void DrawRectangle(char mode, int widht, int height)
{
  char data[] = {
  				7,
				'D',
				'R',
				mode,
				widht>>8,
				widht,
				height>>8,
				height,
				};

  iLCD_CMD(data);
}//end




//****************************************************************************
void DrawLine(int end_x, int end_y)
{
  char data[] = {
  				6			,
				'D'			,
				'L'			,
				end_x>>8	,
				end_x		,
				end_y>>8	,
				end_y		,
				};

  iLCD_CMD(data);
}//end




//****************************************************************************
// Farbscala von 0 - 255, wird allerdings von iLCD reduziert.
void SetForegroundColor(char red, char green, char blue)
{
  char data[] = {
  				6		,
				'A'		,
				'C'		,
				'F'		,
				red		,
				green	,
				blue	,
				};

  iLCD_CMD(data);
}//end




//****************************************************************************
// Farbscala von 0 - 255, wird allerdings von iLCD reduziert.
void SetBackgroundColor(char red, char green, char blue)
{
  
  char data[] = {
  				6		,
				'A'		,
				'C'		,
				'B'		,
				red		,
				green	,
				blue	,
				};

  iLCD_CMD(data);
}//end




//****************************************************************************
void SetScreenOriantation(char oriantation)
{
  char data[] = {
  					2			,
				 	'C'			,
					oriantation	,
				};

  iLCD_CMD(data);
}//end




//****************************************************************************
void InvertScreen(void)
{
  char data[] = {
  					2			,
				 	'D'			,
					'I'			,
				};

  iLCD_CMD(data);
}//end


//****************************************************************************
void DefineViewport(char viewport, char oriantation, int width, int hight )
{
  char data[] = {
  					9			,
				 	'C'			,
					'V'			,
					'D'			,
					viewport	,
					oriantation	,
					width>>8	,
					width		,
					hight>>8	,
					hight		,
				};

  iLCD_CMD(data);
}//end



//****************************************************************************
void SelectViewport(char viewport)
{
  char data[] = {
  					4			,
				 	'C'			,
					'V'			,
					'S'			,
					viewport	,
				};

  iLCD_CMD(data);
}//end




//****************************************************************************
void SetFont(int font)
{
  char data[] = {
  					4			,
				 	'A'			,
					'F'			,
					font<<8		,
					font		,
				};

  iLCD_CMD(data);
}//end



//****************************************************************************
void SetBorderColor(char red, char green, char blue)
{
  
  char data[] = {
  				6		,
				'A'		,
				'C'		,
				'R'		,
				red		,
				green	,
				blue	,
				};

  iLCD_CMD(data);
}//end



//****************************************************************************
void SetTextAlignment(char mode, int widht, int height )
{
  char data[] = {
  					7			,
				 	'C'			,
					'T'			,
					mode		,
					widht>>8	,
					widht		,
					height>>8	,
					height		,	
				};

  iLCD_CMD(data);
}//end





//****************************************************************************
void EraseDisplay(void)
{
  char data[] = {
  					7			,
				 	'D'			,
					'E'			,
				};

  iLCD_CMD(data);
}//end





//****************************************************************************
void EraseDisplayArea(int widht, int height )
{
  char data[] = {
  					6			,
				 	'D'			,
					'e'			,
					widht>>8	,
					widht		,
					height>>8	,
					height		,	
				};

  iLCD_CMD(data);
}//end



